@@ -27,11 +27,10 @@
  */
 
 #include "spellchecklineedit.h"
-
+#include <QStyleOptionFrame>
 #include <QKeyEvent>
 #include <QStyle>
 #include <QApplication>
-#include <QStyleOptionFrameV2>
 #include <QMimeData>
 
 using namespace PimCommon;
@@ -61,9 +60,9 @@ SpellCheckLineEdit::~SpellCheckLineEdit()
 
 void SpellCheckLineEdit::keyPressEvent(QKeyEvent *e)
 {
-    if (e->key() == Qt::Key_Enter ||
-            e->key() == Qt::Key_Return ||
-            e->key() == Qt::Key_Down) {
+    if (e->key() == Qt::Key_Enter
+        || e->key() == Qt::Key_Return
+        || e->key() == Qt::Key_Down) {
         Q_EMIT focusDown();
         return;
     } else if (e->key() == Qt::Key_Up) {
